/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <crown/CrownUtil.hpp>
#include <pelib/Scalar.hpp>
#include <pelib/Vector.hpp>
#include <pelib/Set.hpp>
#include <pelib/Matrix.hpp>

#if 0
#define debug(expr) cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #expr << " = \"" << expr << "\"." << endl;
#else
#define debug(var)
#endif

using namespace std;

namespace pelib
{
namespace crown
{

const int b = 2;
typedef map<int, float> TaskWidthVector;
typedef map<int, float> TaskScheduleVector;
typedef map<int, TaskScheduleVector> ProcScheduleMatrix;

static
int
group_size(int group, int p, int b)
{
	return p / (int)pow(b, floor((log(group) / log(b))));
}

Algebra
CrownUtil::crownToSchedule(const Algebra &crown)
{
	Algebra res = crown;
	//int n = crown.find<Scalar<float> >("n")->getValue();
	int p = (int)crown.find<Scalar<float> >("p")->getValue();
	const Vector<int, float> *group = crown.find<Vector<int, float> >("group");
	//const Vector<int, float> *frequency = crown.find<Vector<int, float> >("frequency");
	vector<vector<int> > groups;
	TaskWidthVector task_width;
	ProcScheduleMatrix proc_schedule;

	// Create a collection of groups, each empty
	for(int group = 1; group <= 2 * p - 1; group++)
	{
		groups.push_back(vector<int>());
	}

	debug(p);	
	// Get the allocated width of each task
	for(map<int, float>::const_iterator i = group->getValues().begin(); i != group->getValues().end(); i++)
	{
		int task = i->first;
		int group = (int)i->second;

		// Get the width of the task
		task_width.insert(pair<int, float>(task, group_size(group, p, b)));

		// Insert the task in the relevant group
		groups[group - 1].push_back(task);
		debug(task);
		debug(group);
	}

	// Get the task sequence for each processor
	int max = 0; // This will hold the maximum number of tasks a core handles
	for(int proc = 0; proc < p; proc++)
	{
		debug(proc);
		int task_number = 0; // This will hold the number of tasks this core holds, may be the max
		TaskScheduleVector schedule;

		for(int lvl = 0; lvl <= log2(p); lvl++)
		{
			int group = (int)(pow((double)b, (double)lvl) + (int)floor(proc / (p / pow((double)b, (double)lvl))));

			debug(proc);
			debug(lvl);
			debug(group);
			vector<int> tasks = groups[group - 1];
			for(vector<int>::iterator iter = tasks.begin(); iter != tasks.end(); iter++)
			{
				debug(proc);
				debug(lvl);
				debug(group);
				debug(task_number);
				debug(*iter);
				schedule.insert(pair<int, float>(task_number, *iter));

				task_number++;
			}
		}

		if(task_number > max) max = task_number; // If that core runs the most amount of tasks so far, then its size is the temporary max
		proc_schedule.insert(pair<int, TaskScheduleVector>(proc + 1, schedule));
	}

	// Fill smaller rows with task 0 so the matrix looks like a proper rectangle matrix
	for(int proc = 0; proc < p; proc++)
	{
		int size = proc_schedule[proc + 1].size();
		debug(proc);
		debug(size);

		while(size < max)
		{
			size++;
			proc_schedule[proc + 1].insert(pair<int, float>(size, 0));
		}
	}

	Matrix<int, int, float> procSchedule = Matrix<int, int, float>("schedule", proc_schedule);
	res.insert(&procSchedule);

	return res;
}

} // namespace crown
} // namespace pelib
